/*
  FUSE: Filesystem in Userspace
  Copyright (C) 2001-2007  Miklos Szeredi <miklos@szeredi.hu>

  This program can be distributed under the terms of the GNU GPL.
  See the file COPYING.

  gcc -Wall `pkg-config fuse --cflags --libs` hello.c -o hello
*/

#define FUSE_USE_VERSION 26

#include <fuse.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <stdlib.h>

static int hello_getattr(const char *path, struct stat *stbuf)
{
	int res = 0;

	memset(stbuf, 0, sizeof(struct stat));
	if (strcmp(path, "/") == 0) {
		stbuf->st_mode = S_IFDIR | 0755;
		stbuf->st_nlink = 2;
	} else {
		char cmd[320];

		snprintf(cmd, sizeof(cmd), "ip link show %s", path + 1);
		res = system(cmd);
		if (res == 0) {
			stbuf->st_mode = S_IFREG | 0444;
			stbuf->st_nlink = 1;
			stbuf->st_size = 32;
		} else
			res = -ENOENT;
	}

	return res;
}

static int hello_readdir(const char *path, void *buf, fuse_fill_dir_t filler,
			 off_t offset, struct fuse_file_info *fi)
{
	FILE *f;
	char ifname[256];
	(void) offset;
	(void) fi;

	if (strcmp(path, "/") != 0)
		return -ENOENT;

	filler(buf, ".", NULL, 0);
	filler(buf, "..", NULL, 0);
	f = popen("ip link show | grep '^[0-9]' | cut -f2 -d:", "r");
	if (f == NULL)
		return -errno;

	while (fscanf(f, "%255s", ifname) == 1)
		filler(buf, ifname, NULL, 0);

	pclose(f);

	return 0;
}

static int hello_open(const char *path, struct fuse_file_info *fi)
{
	if ((fi->flags & 3) != O_RDONLY)
		return -EACCES;

	return 0;
}

static unsigned long long get_interface_bytes(const char *ifname)
{
	FILE *f;
	char cmd[320];
	int res;
	unsigned long long rxbytes, txbytes;

	snprintf(cmd, sizeof(cmd), "ip -s link show %s | sed -nr 's/^ +([0-9]+).*/\\1/p'", ifname);
	f = popen(cmd, "r");
	if (f == NULL)
		return -1;

	res = fscanf(f, "%llu\n%llu\n", &rxbytes, &txbytes);
	pclose(f);
	if (res != 2)
		return -1;

	return rxbytes + txbytes;
}

static int hello_read(const char *path, char *buf, size_t size, off_t offset,
		      struct fuse_file_info *fi)
{
	size_t len;
	(void) fi;
	char num_str[32];
	unsigned long long total_bytes;

	if (strlen(path) > 256)
		return -EIO;

	total_bytes = get_interface_bytes(path + 1);
	if (total_bytes == (unsigned long long) -1)
		return -EIO;

	len = snprintf(num_str, sizeof(num_str), "%llu\n", total_bytes);
	if (offset < len) {
		if (offset + size > len)
			size = len - offset;
		memcpy(buf, num_str + offset, size);
	} else
		size = 0;

	return size;
}

static struct fuse_operations hello_oper = {
	.getattr	= hello_getattr,
	.readdir	= hello_readdir,
	.open		= hello_open,
	.read		= hello_read,
};

int main(int argc, char *argv[])
{
	return fuse_main(argc, argv, &hello_oper, NULL);
}
